"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const builtins = require("./builtins");
const compiledSettingMarker = Symbol();
const defaultOptions = {
    url: {
        requireSchemes: true,
        schemes: ['http', 'https'],
        attributes: {},
    },
};
/**
 * Compiled options.
 */
function compile(transforms, options) {
    const filledOptions = {
        url: Object.assign({}, defaultOptions.url, (options ? options.url : {})),
    };
    if (transforms == null) {
        //default
        transforms = ['url'];
    }
    const compiledTransforms = transforms.map(t => compileTransform(t, filledOptions));
    return {
        [compiledSettingMarker]: true,
        options: filledOptions,
        transforms: compiledTransforms,
    };
}
exports.compile = compile;
/**
 * Compile given setting.
 */
function compileTransform(transform, options) {
    const t = 'string' === typeof transform ? builtins[transform] : transform;
    const pattern = t.pattern(options);
    return {
        pattern,
        transform: t.transform,
    };
}
exports.compileTransform = compileTransform;
/**
 * Check whether given object is a compiled options.
 */
function isCompiledOptions(options) {
    return options != null && options[compiledSettingMarker] === true;
}
exports.isCompiledOptions = isCompiledOptions;
